# @dynatrace-sdk/client-query

[![npm](https://img.shields.io/badge/npm-v1.20.2-blue)](https://www.npmjs.com/package/@dynatrace-sdk/client-query/v/1.20.2)
[![License](https://img.shields.io/badge/License-Apache_2.0-blue.svg)](https://opensource.org/licenses/Apache-2.0)

Exposes an API to fetch records stored in Grail


## Installation

```bash
npm install @dynatrace-sdk/client-query
```


## Getting help

- Visit [SDK for Typescript](https://developer.dynatrace.com/reference/sdks/) guide in the [Dynatrace Developer](https://developer.dynatrace.com/)
- Ask a question in the [Dynatrace Community](https://community.dynatrace.com/)


## License

This SDK is distributed under the [Apache License, Version 2.0](http://www.apache.org/licenses/LICENSE-2.0), see LICENSE for more information.


# API reference

Full API reference for the latest version of the SDK is also available at the [Dynatrace Developer](https://developer.dynatrace.com/reference/sdks/client-query/).

## queryAssistanceClient

```js
import { queryAssistanceClient } from '@dynatrace-sdk/client-query';
```


<a name="queryautocomplete"></a>
### queryAutocomplete

<div class="padding-bottom--md">
<strong>queryAssistanceClient.queryAutocomplete(config): Promise&lt;<a href="#autocompleteresponse">AutocompleteResponse</a>&gt;</strong>

<div class="padding-left--md">

Get a structured list of suggestions for the query at the given position.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

## Overview

We provide a list of suggestions that may be used after the cursor position. The following queries will all provide the
same results:

* `query: "f"`
* `query: "f", cursorPosition:1`
* `query: "fetch ", cursorPosition:1`

Available fields:

| Field       | Description                                                                                                                                                                             |
|-------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| suggestions | a list of suggestions. Each item is a separate possible suggestion, despite they might have the same outputs.                                                                           |
| optional    | whether the suggestion is optional. If `true`, the query until the cursor position might work. If `false`, the query is definitely incomplete or invalid if cut at the cursor position. |

Fields in the `suggestions`

| Field                  | Description                                                                                                |
|------------------------|------------------------------------------------------------------------------------------------------------|
| suggestion             | a string representing the whole suggestion. This information could also be derived from the parts.         |
| alreadyTypedCharacters | how many characters of this suggestion have already been typed (and will be overridden by the suggestion). |
| parts                  | a list of semantically enriched information on what are the parts of a suggestion.                         |

Fields in `parts`

| Field      | Description                                               |
|------------|-----------------------------------------------------------|
| suggestion | a string representing the current part of the suggestion. |
| type       | current types: SPACE, PIPE, COMMAND (may be extended)     |

The `type` helps to treat specific parts of the suggestion different to others; either by a different visualization,
a link to docs, etc.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.body<sup>*required</sup>|<a href="#autocompleterequest" target="_blank" rel="noopener noreferrer">AutocompleteRequest</a>| |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|config.enforceQueryConsumptionLimit|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|If set, query consumption limit will be enforced. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;AutocompleteResponse&gt;|A list of structured autocomplete suggestions.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. Either the query itself or other parameters are wrong. \| An internal server error has occurred.|


<details>
<summary>
Code example
</summary>

```ts
import { queryAssistanceClient } from "@dynatrace-sdk/client-query";

const data = await queryAssistanceClient.queryAutocomplete({
  body: {
    query:
      'fetch events | filter event.type == "davis" AND davis.status != "CLOSED" | fields timestamp, davis.title, davis.underMaintenance, davis.status | sort timestamp | limit 10',
  },
});
```

</details>



</div>


<a name="queryparse"></a>
### queryParse

<div class="padding-bottom--md">
<strong>queryAssistanceClient.queryParse(config): Promise&lt;<a href="#dqlnode">DQLNode</a>&gt;</strong>

<div class="padding-left--md">

Get a structured tree of the canonical form of the query.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

## Overview

Returns the parsed query as a tree, containing the structure of the canonical query. Tree-nodes can contain references to
the token position where they originate from. This may help to provide hover effects, show canonical forms, mark
optional items, and more.

## Details

The query tree consists of nodes that contain different additional information (everything optional):

### General Fields

Field | Mandatory |  Description
----------------------- | --- | ----------------------------------------------------------------------------------------
tokenPosition           | no  | optional. If present, it represents the position within the query string where the node refers to.
isOptional              | no  | whether this node could be left out and the result would still be the same query (semantically).

#### tokenPosition

contains `start` (inclusive) and `end` (inclusive), both contain `index` (0 based; fur substrings), `line`
and `column` (both 1-based; for readability).

* If `tokenPosition` is present, it always contains start and end with all fields
* If `tokenPosition` is not present, there might still be nested nodes that do contain a position
* If `start == end`, the position refers to a single character
* If `start > end`, we know for sure that something was inserted.

We can always check whether the canonical representation of a node matches the text in the tokenPosition to see whether
something was inserted, removed, or changed.

#### isOptional

only present if it is true.

Optional nodes can e.g. be optional braces that make a query more readable, but are not necessary. This could be used to
enter *ghost braces* and *implicit functions* in the user&apos;s input field; maybe with different formatting
(using the tokenPosition of sibling nodes we can also check whether the user wrote these or not).

### Advanced Token Types

each node is of one of following types and may contain more fields:

* Terminal Node
* ContainerNode
* Alternative Node

#### Terminal Node

can be identified by checking whether `canonicalString` is present

Field | Mandatory |  Description
------------------------ | --- | ---------------------------------------------------------------------------------------
type                     | yes | the type of the terminal node - do not confuse with the type of container nodes
canonicalString          | yes | the canonical string representation. Concatenating the canonicalString of all nested terminal nodes provides the canonical form of the query.
isMandatoryOnUserOrder   | no  | may only be present if (`type="BRACE_OPEN"` or `type="BRACE_CLOSE"`) and `isOptional=true`. For usage see section Special node type: PARAMETERS

##### Current types of terminal nodes (list might grow):

* SPACE
* LINEBREAK
* INDENT
* PIPE
* DOT
* COLON
* COMMA
* BRACE_OPEN
* BRACE_CLOSE
* BRACKET_OPEN
* BRACKET_CLOSE
* PARENTHESIS_OPEN
* PARENTHESIS_CLOSE
* QUOTE
* SLASH
* BOOLEAN_TRUE
* BOOLEAN_FALSE
* NULL
* COMMAND_NAME
* PARAMETER_KEY
* PARAMETER_VALUE_SCOPE
* FUNCTION_NAME
* OPERATOR
* TRAVERSAL_OPERATOR
* TRAVERSAL_RELATION_NAME
* TRAVERSAL_HOP_COUNT
* SIMPLE_IDENTIFIER
* NUMBER
* STRING
* TIME_UNIT
* TIMESTAMP_VALUE
* METRIC_KEY
* VARIABLE

#### ContainerNode

can be identified by checking whether `children` is present

Field | Mandatory |  Description
----------------------- | --- | ----------------------------------------------------------------------------------------
type                    | yes | the type of the container node - do not confuse with the type of terminal nodes
children                | yes | the children for the node. might be of any type

##### Current types of container nodes (list might grow):

* QUERY
* EXECUTION_BLOCK
* COMMAND
* COMMAND_SEPARATOR
* PARAMETER_WITH_KEY
* GROUP
* PARAMETERS - check examples further down
* PARAMETER_NAMING
* PARAMETER_SEPARATOR
* FUNCTION
* FUNCTION_PART - check examples further down
* EXPRESSION
* IDENTIFIER
* SOURCE_ID
* DURATION
* TIMESTAMP
* TIMEFRAME
* TRAVERSAL_PATH
* TRAVERSAL_STEP

##### Special node type: PARAMETERS

can contain children representing the parameters. Every second child is of type PARAMETER_SEPARATOR.

You may reorder the children based on their tokenPosition to get the user order. However, in this case,
you need to consider `isMandatoryOnUserOrder` to determine whether the grouping braces are mandatory or not.

###### Example

For the query `SORT a, {direction:"descending", b}`, the canonical form is:

`SORT a, {b, direction:"descending"}`

This is the order, in which the parameters are returned in the query tree.
Parameters are \{a} and \{\{b} and \{direction:&quot;descending&quot;}}. In this case, the braces are optional.

`SORT a, {b, direction:"descending"}` is equivalent to `SORT a, b, direction:"descending"`

However, if you reorder the children by tokenPosition, the braces are not optional, because

`SORT a, direction:"descending", b` is interpreted as `SORT {a, direction:"descending"}, b`

So, if the children in PARAMETERS are re-ordered by tokenPosition, braces (or in general: TerminalNodes)
are only optional if `isOptional && !isMandatoryOnUserOrder`.

##### Special node type: FUNCTION_PART

A container node of type `FUNCTION` may contain nodes of type `FUNCTION_PART`.

If those `FUNCTION_PART`s are marked as optional, this means you have to either include *all* or *none* of these
optional function parts.

*Example*:

`filter anyMatch(a.b == 1, input:a)`

The optional function parts are `anyMatch(` and `, input:a)`. If you leave out both, the command will still work:
`filter a.b == 1` and return the same result. Using one of these optional function parts and removing the other will lead
to an invalid query.

#### Alternative Node

can be identified by checking whether `alternatives` is present

Field | Mandatory |  Description
----------------------- | --- | ----------------------------------------------------------------------------------------
alternatives            | yes | Type: Map&amp;lt;AlternativeType, DQLNode&amp;gt;

When displaying the query, pick one option. You may use the other options for hovering, replacing, and more.

##### Current values of AlternativeType (list might grow):

* CANONICAL: This node is the one we will use for our canonical form
* USER: An alternative that is also valid, but not canonical; and this version was picked by the user.
* INFO: only if the canonical version is not present

*Examples*:

* `CANONICAL` is not present, `USER` is present: user&apos;s nodes are optional, but not canonical (usually optional nodes
  are still canonical)
* `CANONICAL` is present, `USER` is not present: same as if the canonical node was optional. If this happens, it is
  likely that there is also an `INFO` node
* `CANONICAL` is present, `USER` is present: there are different alternatives
* `INFO` is present: usually if `CANONICAL` is not present (e.g. the parameter key for `FILTER a == 1`), there is an info node
  for `FILTER condition:a == 1`. This `condition:` was neither written by the user nor is it canonical; but it might be
  used to help the user understand what this parameter means.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.body<sup>*required</sup>|<a href="#parserequest" target="_blank" rel="noopener noreferrer">ParseRequest</a>| |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;DQLNode&gt;|A node containing more nodes,  a node offering different (semantically equivalent) versions of the query parts, or a terminal node that shows the canonical form.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. Either the query itself or other parameters are wrong. \| An internal server error has occurred.|


<details>
<summary>
Code example
</summary>

```ts
import { queryAssistanceClient } from "@dynatrace-sdk/client-query";

const data = await queryAssistanceClient.queryParse({
  body: {
    query:
      'fetch events | filter event.type == "davis" AND davis.status != "CLOSED" | fields timestamp, davis.title, davis.underMaintenance, davis.status | sort timestamp | limit 10',
  },
});
```

</details>



</div>


<a name="queryverify"></a>
### queryVerify

<div class="padding-bottom--md">
<strong>queryAssistanceClient.queryVerify(config): Promise&lt;<a href="#verifyresponse">VerifyResponse</a>&gt;</strong>

<div class="padding-left--md">

Verifies a query without executing it.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

## Overview

Verifies the supplied query string and other query parameters for lack of any errors, but without actually
submitting the query for execution.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.body<sup>*required</sup>|<a href="#verifyrequest" target="_blank" rel="noopener noreferrer">VerifyRequest</a>| |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;VerifyResponse&gt;|Supplied query and parameters were verified.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. \| An internal server error has occurred.|


<details>
<summary>
Code example
</summary>

```ts
import { queryAssistanceClient } from "@dynatrace-sdk/client-query";

const data = await queryAssistanceClient.queryVerify({
  body: {
    query:
      'fetch events | filter event.type == "davis" AND davis.status != "CLOSED" | fields timestamp, davis.title, davis.underMaintenance, davis.status | sort timestamp | limit 10',
  },
});
```

</details>



</div>

## queryExecutionClient

```js
import { queryExecutionClient } from '@dynatrace-sdk/client-query';
```


<a name="querycancel"></a>
### queryCancel

<div class="padding-bottom--md">
<strong>queryExecutionClient.queryCancel(config): Promise&lt;void &#124; <a href="#querypollresponse">QueryPollResponse</a>&gt;</strong>

<div class="padding-left--md">

Cancels the query and returns the result if the query was already finished, otherwise discards it.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

### Overview:

Cancels a running Grail query and returns a list of records if the query already finished.

### The response format:

If the query was already finished, a response body including the result will be returned. Otherwise the response will contain no body.

The result has three main sections:
* the &apos;records&apos; section contains the individual records, where each record consists of a set of fields and their corresponding values.
* the &apos;types&apos; section describes the corresponding data types that a record field has.
* the &apos;metadata&apos; section contains information about the query like &apos;analysisTimeframe&apos;, &apos;timezone&apos; or &apos;locale&apos;.

Every record has an implicit &apos;index&apos; according to the position in the &apos;records&apos; JSON array.
The types section has a list of 1..N possible type &apos;buckets&apos;. Each such bucket has an &apos;indexRange&apos; which indicates which
records will find their field types in which bucket. The index range has two values start &amp; end and can be thought of as [startIndex, endIndex).

A field part of a record with index &apos;i&apos; will find its corresponding field type by first locating the bucket that satisfies:

```text
startIndex <= i <= endIndex
```

Once the bucket is found the &apos;mappings&apos; object has an entry for all the fields that are part of that record with index &apos;i&apos;.


Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
field name but their values are of a different type. This phenomenon will hence forth be named as a **&quot;collision&quot;**.
When a collision does occur, we will create a new type &apos;bucket&apos; that will have a different index range where
the new record field types will be placed.
It is guaranteed that every field of every record will have a corresponding type.
**Clients should always take the included types into account when consuming records!**



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|config.enrich|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|If set additional data will be available in the metadata section. |
|config.requestToken<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The request-token of the query. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;QueryPollResponse \| void&gt;|The query already finished.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. Either the query itself or other parameters are wrong. \| An internal server error has occurred.|


<details>
<summary>
Code example
</summary>

```ts
import { queryExecutionClient } from "@dynatrace-sdk/client-query";

const data = await queryExecutionClient.queryCancel({
  requestToken: "...",
});
```

</details>



</div>


<a name="queryexecute"></a>
### queryExecute

<div class="padding-bottom--md">
<strong>queryExecutionClient.queryExecute(config): Promise&lt;<a href="#querystartresponse" target="_blank" rel="noopener noreferrer">QueryStartResponse</a>&gt;</strong>

<div class="padding-left--md">

Starts a Grail query.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

### Overview:

Executes a query and returns a list of records.

For details about the query language see the [Dynatrace Query Language documentation](https://dt-url.net/bv03yk8).

### The response format:

The json response will contain the state of the started query. If the query succeeded, the result will be included. Otherwise the response will contain a request token to reference the query in future polling requests.

The result has two main sections:
* The &apos;records&apos; section contains the individual records, where each record consists of a set of fields and their corresponding values.
* The &apos;types&apos; section describes the corresponding data types that a record field has.

Every record has an implicit &apos;index&apos; according to the position in the &apos;records&apos; JSON array.
The types section has a list of 1..N possible type &apos;buckets&apos;. Each such bucket has an &apos;indexRange&apos; which indicates which
records will find their field types in which bucket. The index range has two values start &amp; end and can be thought of as [startIndex, endIndex).

A field part of a record with index &apos;i&apos; will find its corresponding field type by first locating the bucket that satisfies:

```text
startIndex <= i <= endIndex
```

Once the bucket is found the &apos;mappings&apos; object has an entry for all the fields that are part of that record with index &apos;i&apos;.


Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
field name but their values are of a different type. This phenomenon will hence forth be named as a **&quot;collision&quot;**.
When a collision does occur, we will create a new type &apos;bucket&apos; that will have a different index range where
the new record field types will be placed.
It is guaranteed that every field of every record will have a corresponding type.
**Clients should always take the included types into account when consuming records!**



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.body<sup>*required</sup>|<a href="#executerequest">ExecuteRequest</a>| |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|config.enforceQueryConsumptionLimit|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|If set, query consumption limit will be enforced. |
|config.enrich|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|If set additional data will be available in the metadata section. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;QueryStartResponse&gt;|The final status and results of the supplied query if it finished within a supplied requestTimeoutMilliseconds. \| The status of the query to start.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. Either the query itself or other parameters are wrong. \| Too many requests. \| An internal server error has occurred. \| Service is unavailable. \| Client error. \| Server error.|
|InsufficientPermission|Insufficient permissions.|


<details>
<summary>
Code example
</summary>

```ts
import { queryExecutionClient } from "@dynatrace-sdk/client-query";

const data = await queryExecutionClient.queryExecute({
  body: {
    query:
      'fetch events | filter event.type == "davis" AND davis.status != "CLOSED" | fields timestamp, davis.title, davis.underMaintenance, davis.status | sort timestamp | limit 10',
  },
});
```

</details>



</div>


<a name="querypoll"></a>
### queryPoll

<div class="padding-bottom--md">
<strong>queryExecutionClient.queryPoll(config): Promise&lt;<a href="#querypollresponse" target="_blank" rel="noopener noreferrer">QueryPollResponse</a>&gt;</strong>

<div class="padding-left--md">

Retrieves query status and final result from Grail.

For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).

### Overview:

Polls the status of a Grail query. Returns the status of the query, including the result if the query finished.

### The response format:
The json response will contain the state of the query. If the query succeeded, the result will be included.

The result has two main sections:
* The &apos;records&apos; section contains the individual records, where each record consists of a set of fields and their corresponding values.
* The &apos;types&apos; section describes the corresponding data types that a record field has.

Every record has an implicit &apos;index&apos; according to the position in the &apos;records&apos; JSON array.
The types section has a list of 1..N possible type &apos;buckets&apos;. Each such bucket has an &apos;indexRange&apos; which indicates which
records will find their field types in which bucket. The index range has two values start &amp; end and can be thought of as [startIndex, endIndex).

A field part of a record with index &apos;i&apos; will find its corresponding field type by first locating the bucket that satisfies:

```text
startIndex <= i <= endIndex
```

Once the bucket is found the &apos;mappings&apos; object has an entry for all the fields that are part of that record with index &apos;i&apos;.


Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
field name but their values are of a different type. This phenomenon will hence forth be named as a **&quot;collision&quot;**.
When a collision does occur, we will create a new type &apos;bucket&apos; that will have a different index range where
the new record field types will be placed.
It is guaranteed that every field of every record will have a corresponding type.
**Clients should always take the included types into account when consuming records!**



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|config.dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|config.enrich|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|If set additional data will be available in the metadata section. |
|config.requestTimeoutMilliseconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The time a client is willing to wait for the query result. In case the query finishes within the specified timeout, the query result is returned. Otherwise, the query status is returned. |
|config.requestToken<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The request-token of the query. |




#### Returns

| Return type | Description |
|---|---|
|Promise&lt;QueryPollResponse&gt;|The current status and results of the supplied query.|


#### Throws

| Error Type | Error Message |
|---|---|
|ErrorEnvelopeError|The supplied request is wrong. Either the query itself or other parameters are wrong. \| An internal server error has occurred.|


<details>
<summary>
Code example
</summary>

```ts
import { queryExecutionClient } from "@dynatrace-sdk/client-query";

const data = await queryExecutionClient.queryPoll({
  requestToken: "...",
});
```

</details>



</div>

## Types

### AutocompleteRequest

| Name | Type | Description |
| --- | --- | --- |
|cursorPosition|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)| |
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|maxDataSuggestions|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)| |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The full query string. |
|queryOptions|<a href="#queryoptions">QueryOptions</a>|Query options enhance query functionality for Dynatrace internal services. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |

### AutocompleteResponse

The response of the autocomplete call.

| Name | Type | Description |
| --- | --- | --- |
|optional<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the suggestions are optional. |
|suggestedTtlSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Suggested duration in seconds, for how long the response may be cached and reused by the client. It is derived from the volatility of the suggestions on the server (if the suggestions are static, how long the server will cache the volatile suggestions, ...). If not provided, then the result may be cached for long time. Value below 1 means that the result should not be cached. |
|suggestions<sup>*required</sup>|Array&lt;<a href="#autocompletesuggestion" target="_blank" rel="noopener noreferrer">AutocompleteSuggestion</a>&gt;|The list of suggestions. |

### AutocompleteSuggestion

Single suggestion for completion of the query.

| Name | Type | Description |
| --- | --- | --- |
|alreadyTypedCharacters<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Number of characters that the user already typed for this suggestion. |
|parts<sup>*required</sup>|Array&lt;<a href="#autocompletesuggestionpart">AutocompleteSuggestionPart</a>&gt;|List of suggestion parts. |
|suggestion<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The suggested continuation of the query. |

### AutocompleteSuggestionPart

Part of the suggestion.

| Name | Type | Description |
| --- | --- | --- |
|info|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The type of the suggestion. |
|suggestion<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The suggested continuation of the query. |
|synopsis|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The synopsis of the suggestion. |
|type<sup>*required</sup>|"SPACE" &#124; "LINEBREAK" &#124; "INDENT" &#124; "PIPE" &#124; "DOT" &#124; "COLON" &#124; "COMMA" &#124; "ASSIGNMENT" &#124; "BRACE_OPEN" &#124; "BRACE_CLOSE" &#124; "BRACKET_OPEN" &#124; "BRACKET_CLOSE" &#124; "PARENTHESIS_OPEN" &#124; "PARENTHESIS_CLOSE" &#124; "QUOTE" &#124; "SINGLE_QUOTE" &#124; "SLASH" &#124; "BOOLEAN_TRUE" &#124; "BOOLEAN_FALSE" &#124; "NULL" &#124; "COMMAND_NAME" &#124; "PARAMETER_KEY" &#124; "PARAMETER_VALUE_SCOPE" &#124; "FUNCTION_NAME" &#124; "TIMESERIES_AGGREGATION" &#124; "TIMESERIES_AGGREGATION_EXPRESSION" &#124; "OPERATOR" &#124; "SEARCH_OPERATOR" &#124; "TRAVERSAL_OPERATOR" &#124; "TRAVERSAL_RELATION_NAME" &#124; "TRAVERSAL_HOP_COUNT" &#124; "SIMPLE_IDENTIFIER" &#124; "DATA_OBJECT" &#124; "NUMBER" &#124; "STRING" &#124; "TIME_UNIT" &#124; "TIMESTAMP_VALUE" &#124; "METRIC_KEY" &#124; "VARIABLE" &#124; "END_COMMENT" &#124; "UID_VALUE" &#124; "PARSE_PATTERN" &#124; "FIELD_PATTERN" &#124; "SEARCH_PATTERN" &#124; "ENTITY_SELECTOR_PART" &#124; "SAVED_TABLE_NAME" &#124; "PARAMETER_MODIFIER" &#124; "FIELD_MODIFIER" &#124; "ENTITY_TYPE" &#124; "ENTITY_ATTRIBUTE" &#124; "SMARTSCAPE_ID_VALUE"|The type of the autocomplete token. |

### DQLAlternativeNode

The DQL node that has alternatives.

| Name | Type | Description |
| --- | --- | --- |
|alternatives|object|The different alternatives. |
|isOptional<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the node is optional. |
|nodeType<sup>*required</sup>|"TERMINAL" &#124; "CONTAINER" &#124; "ALTERNATIVE"|The type of the node. |
|tokenPosition|<a href="#tokenposition" target="_blank" rel="noopener noreferrer">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |

### DQLContainerNode

The DQL node that contains other nodes.

| Name | Type | Description |
| --- | --- | --- |
|children|Array&lt;<a href="#dqlnode">DQLNode</a>&gt;|The list of children nodes. |
|isOptional<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the node is optional. |
|nodeType<sup>*required</sup>|"TERMINAL" &#124; "CONTAINER" &#124; "ALTERNATIVE"|The type of the node. |
|tokenPosition|<a href="#tokenposition" target="_blank" rel="noopener noreferrer">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |
|type|"QUERY" &#124; "EXECUTION_BLOCK" &#124; "COMMAND" &#124; "COMMAND_SEPARATOR" &#124; "PARAMETER_WITH_KEY" &#124; "GROUP" &#124; "PARAMETERS" &#124; "PARAMETER_NAMING" &#124; "PARAMETER_ASSIGNMENT" &#124; "PARAMETER_SEPARATOR" &#124; "FUNCTION" &#124; "FUNCTION_PART" &#124; "EXPRESSION" &#124; "IDENTIFIER" &#124; "SOURCE_ID" &#124; "DURATION" &#124; "TIMESTAMP" &#124; "TIMEFRAME" &#124; "TRAVERSAL_PATH" &#124; "TRAVERSAL_STEP"|The type of the node. |

### DQLNode

General Node in the DQL query.

| Name | Type | Description |
| --- | --- | --- |
|isOptional<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the node is optional. |
|nodeType<sup>*required</sup>|"TERMINAL" &#124; "CONTAINER" &#124; "ALTERNATIVE"|The type of the node. |
|tokenPosition|<a href="#tokenposition">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |

### DQLTerminalNode

Node that represents single token.

| Name | Type | Description |
| --- | --- | --- |
|canonicalString|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Canonical form. |
|isMandatoryOnUserOrder|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|For optional items only: whether this node becomes mandatory when user order is used. True only for some optional 'ghost braces' |
|isOptional<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the node is optional. |
|nodeType<sup>*required</sup>|"TERMINAL" &#124; "CONTAINER" &#124; "ALTERNATIVE"|The type of the node. |
|tokenPosition|<a href="#tokenposition" target="_blank" rel="noopener noreferrer">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |
|type|"SPACE" &#124; "LINEBREAK" &#124; "INDENT" &#124; "PIPE" &#124; "DOT" &#124; "COLON" &#124; "COMMA" &#124; "ASSIGNMENT" &#124; "BRACE_OPEN" &#124; "BRACE_CLOSE" &#124; "BRACKET_OPEN" &#124; "BRACKET_CLOSE" &#124; "PARENTHESIS_OPEN" &#124; "PARENTHESIS_CLOSE" &#124; "QUOTE" &#124; "SINGLE_QUOTE" &#124; "SLASH" &#124; "BOOLEAN_TRUE" &#124; "BOOLEAN_FALSE" &#124; "NULL" &#124; "COMMAND_NAME" &#124; "PARAMETER_KEY" &#124; "PARAMETER_VALUE_SCOPE" &#124; "FUNCTION_NAME" &#124; "TIMESERIES_AGGREGATION" &#124; "TIMESERIES_AGGREGATION_EXPRESSION" &#124; "OPERATOR" &#124; "SEARCH_OPERATOR" &#124; "TRAVERSAL_OPERATOR" &#124; "TRAVERSAL_RELATION_NAME" &#124; "TRAVERSAL_HOP_COUNT" &#124; "SIMPLE_IDENTIFIER" &#124; "DATA_OBJECT" &#124; "NUMBER" &#124; "STRING" &#124; "TIME_UNIT" &#124; "TIMESTAMP_VALUE" &#124; "METRIC_KEY" &#124; "VARIABLE" &#124; "END_COMMENT" &#124; "UID_VALUE" &#124; "PARSE_PATTERN" &#124; "FIELD_PATTERN" &#124; "SEARCH_PATTERN" &#124; "ENTITY_SELECTOR_PART" &#124; "SAVED_TABLE_NAME" &#124; "PARAMETER_MODIFIER" &#124; "FIELD_MODIFIER" &#124; "ENTITY_TYPE" &#124; "ENTITY_ATTRIBUTE" &#124; "SMARTSCAPE_ID_VALUE"|The type of the autocomplete token. |

### ErrorEnvelope

An &apos;envelope&apos; error object that has the mandatory error object.

| Name | Type | Description |
| --- | --- | --- |
|error<sup>*required</sup>|<a href="#errorresponse">ErrorResponse</a>|The response for error state |

### ErrorResponse

The response for error state

| Name | Type | Description |
| --- | --- | --- |
|code<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Error code, which normally matches the HTTP error code. |
|details|<a href="#errorresponsedetails" target="_blank" rel="noopener noreferrer">ErrorResponseDetails</a>|Detailed information about the error. |
|message<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|A short, clear error message without details |

### ErrorResponseDetails

Detailed information about the error.

| Name | Type | Description |
| --- | --- | --- |
|arguments|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|The arguments for the message format. |
|constraintViolations|Array&lt;<a href="#errorresponsedetailsconstraintviolationsitem">ErrorResponseDetailsConstraintViolationsItem</a>&gt;|Information about an input parameter that violated some validation rule of the service API. |
|errorMessage|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Complete error message. |
|errorMessageFormat|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The message format of the error message, string.format based. |
|errorMessageFormatSpecifierTypes|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|The corresponding DQL format specifier types for each format specifier used in the error message format. |
|errorType|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The error type, e.g. COMMAND_NAME_MISSING |
|exceptionType|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The exception type. |
|missingPermissions|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|List of missing IAM permissions necessary to successfully execute the request. |
|missingScopes|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|List of missing IAM scopes necessary to successfully execute the request. |
|queryId|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| |
|queryString|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Submitted query string. |
|syntaxErrorPosition|<a href="#tokenposition" target="_blank" rel="noopener noreferrer">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |

### ErrorResponseDetailsConstraintViolationsItem

| Name | Type | Description |
| --- | --- | --- |
|message<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Message describing the error. |
|parameterDescriptor|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Describes the violating parameter. |
|parameterLocation|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Describes the general location of the violating parameter. |

### ExecuteRequest

| Name | Type | Description |
| --- | --- | --- |
|defaultSamplingRatio|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|In case not specified in the DQL string, the sampling ratio defined here is applied. Note that this is only applicable to log queries. |
|defaultScanLimitGbytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Limit in gigabytes for the amount data that will be scanned during read. |
|defaultTimeframeEnd|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'end' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'start' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|defaultTimeframeStart|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'start' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'end' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|enablePreview|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Request preview results. If a preview is available within the requestTimeoutMilliseconds, then it will be returned as part of the response. |
|fetchTimeoutSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The query will stop reading data after reaching the fetch-timeout. The query execution will continue, providing a partial result based on the read data. |
|filterSegments|<a href="#filtersegments">FilterSegments</a>|Represents a collection of filter segments. |
|includeTypes|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Parameter to exclude the type information from the query result. In case not specified, the type information will be included. <i>default:</i> <code>true</code>|
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|maxResultBytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result bytes that this query will return. |
|maxResultRecords|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result records that this query will return. |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The full query string. |
|queryOptions|<a href="#queryoptions" target="_blank" rel="noopener noreferrer">QueryOptions</a>|Query options enhance query functionality for Dynatrace internal services. |
|requestTimeoutMilliseconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum time the response will be delayed to wait for a result. (This excludes the sending time and time spent in any services between the query-frontend and the client.) If the query finishes within the specified timeout, the query result is returned. Otherwise, the requestToken is returned, allowing polling for the result. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |

### FieldType

The possible type of a field in DQL.

| Name | Type |
| --- | --- |
|type<sup>*required</sup>|"string" &#124; "boolean" &#124; "undefined" &#124; "binary" &#124; "double" &#124; "long" &#124; "timestamp" &#124; "timeframe" &#124; "duration" &#124; "ip_address" &#124; "geo_point" &#124; "array" &#124; "record" &#124; "uid" &#124; "smartscape_id"| 
|types|Array&lt;<a href="#rangedfieldtypes">RangedFieldTypes</a>&gt;|

### FilterSegment

A filter segment is identified by an ID. Each segment includes a list of variable definitions.

| Name | Type |
| --- | --- |
|id<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| 
|variables|Array&lt;<a href="#filtersegmentvariabledefinition" target="_blank" rel="noopener noreferrer">FilterSegmentVariableDefinition</a>&gt;|

### FilterSegmentVariableDefinition

Defines a variable with a name and a list of values.

| Name | Type |
| --- | --- |
|name<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| 
|values<sup>*required</sup>|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|

### FilterSegments

Represents a collection of filter segments.

**type**: Array&lt;<a href="#filtersegment">FilterSegment</a>&gt;

### GeoPoint

DQL data type representing a geolocation point.

| Name | Type | Description |
| --- | --- | --- |
|latitude<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The coordinate that specifies the north-south position of a point on the surface of the earth. |
|longitude<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The coordinate that specifies the  east-west position of a point on the surface of the earth. |

### GrailMetadata

Collects various bits of metadata information.

| Name | Type | Description |
| --- | --- | --- |
|analysisTimeframe|<a href="#timeframe" target="_blank" rel="noopener noreferrer">Timeframe</a>|DQL data type timeframe. |
|canonicalQuery|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The canonical form of the query. It has normalized spaces and canonical constructs. |
|dqlVersion|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The version of DQL that was used to process the query request. |
|executionTimeMilliseconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The time it took to execute the query. |
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Effective locale for the query. |
|notifications|Array&lt;<a href="#metadatanotification">MetadataNotification</a>&gt;|Collected messages during the execution of the query. |
|query|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The submitted query. |
|queryId|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The id of the query |
|sampled|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if sampling was used for at least one segment. |
|scannedBytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Number of scanned bytes during the query execution. |
|scannedDataPoints|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)| |
|scannedRecords|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Number of scanned records during the query execution. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Effective timezone for the query. |

### Metadata

Collects various bits of metadata information.

| Name | Type | Description |
| --- | --- | --- |
|grail|<a href="#grailmetadata" target="_blank" rel="noopener noreferrer">GrailMetadata</a>|Collects various bits of metadata information. |
|metrics|Array&lt;<a href="#metricmetadata">MetricMetadata</a>&gt;| |

### MetadataNotification

The message that provides additional information about the execution of the query.

| Name | Type | Description |
| --- | --- | --- |
|arguments|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|The arguments for the message format. |
|message|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The complete message of the notification. |
|messageFormat|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The message format of the notification, string.format based |
|messageFormatSpecifierTypes|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;|The corresponding DQL format specifier types for each format specifier used in the error message format. |
|notificationType|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The notification type, e.g. LIMIT_ADDED. |
|severity|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The severity of the notification, currently: INFO, WARN, ERROR. |
|syntaxPosition|<a href="#tokenposition" target="_blank" rel="noopener noreferrer">TokenPosition</a>|The position of a token in a query string used for errors and notification to map the message to a specific part of the query. |

### MetricMetadata

An object that defines additional metric metadata.

| Name | Type | Description |
| --- | --- | --- |
|description|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The description of the metadata. |
|displayName|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The display name of the metadata. |
|fieldName|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The name of the associated field used in the query. |
|metric.key|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The metric key. |
|rate|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The specified rate normalization parameter. |
|rollup|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|Metadata about the rollup type. |
|scalar|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Indicates whether the scalar parameter was set to true in the timeseries aggregation function. |
|shifted|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Indicates if the shifted parameter was used. |
|unit|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The unit used. |

### ParseRequest

| Name | Type | Description |
| --- | --- | --- |
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The full query string. |
|queryOptions|<a href="#queryoptions">QueryOptions</a>|Query options enhance query functionality for Dynatrace internal services. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |

### PositionInfo

The exact position in the query string.

| Name | Type | Description |
| --- | --- | --- |
|column<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Query position column zero based index. |
|index<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Query position index. |
|line<sup>*required</sup>|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Query position line zero based index. |

### QueryOptions

Query options enhance query functionality for Dynatrace internal services.

**type**: Record&lt;string, string | undefined&gt;

### QueryPollResponse

The response of GET query:execute call.

| Name | Type | Description |
| --- | --- | --- |
|progress|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The progress of the query from 0 to 100. |
|result|<a href="#queryresult" target="_blank" rel="noopener noreferrer">QueryResult</a>|The result of the DQL query. |
|state<sup>*required</sup>|"NOT_STARTED" &#124; "RUNNING" &#124; "SUCCEEDED" &#124; "RESULT_GONE" &#124; "CANCELLED" &#124; "FAILED"|Possible state of the query. |
|ttlSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Time to live in seconds. |

### QueryResult

The result of the DQL query.

| Name | Type | Description |
| --- | --- | --- |
|metadata<sup>*required</sup>|<a href="#metadata">Metadata</a>|Collects various bits of metadata information. |
|records<sup>*required</sup>|Array&lt;null &#124; <a href="#resultrecord" target="_blank" rel="noopener noreferrer">ResultRecord</a>&gt;|List of records containing the result fields data. |
|types<sup>*required</sup>|Array&lt;<a href="#rangedfieldtypes">RangedFieldTypes</a>&gt;|The data types for the result records. |

### QueryStartResponse

The response when starting a query.

| Name | Type | Description |
| --- | --- | --- |
|progress|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The progress of the query from 0 to 100. |
|requestToken|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The token returned by the POST query:execute call. |
|result|<a href="#queryresult" target="_blank" rel="noopener noreferrer">QueryResult</a>|The result of the DQL query. |
|state<sup>*required</sup>|"NOT_STARTED" &#124; "RUNNING" &#124; "SUCCEEDED" &#124; "RESULT_GONE" &#124; "CANCELLED" &#124; "FAILED"|Possible state of the query. |
|ttlSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Time to live in seconds. |

### RangedFieldTypes

The field type in range.

| Name | Type | Description |
| --- | --- | --- |
|indexRange|Array&lt;[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)&gt;|The range of elements at use this type in arrays (null for records). |
|mappings<sup>*required</sup>|<a href="#rangedfieldtypesmappings">RangedFieldTypesMappings</a>|The mapping between the field name and data type. |

### RangedFieldTypesMappings

The mapping between the field name and data type.

**type**: Record&lt;string, FieldType | undefined&gt;

### ResultRecord

Single record that contains the result fields.

**type**: Record&lt;string, ResultRecordValue | null | undefined&gt;

### Timeframe

DQL data type timeframe.

| Name | Type | Description |
| --- | --- | --- |
|end|Date|The end time of the timeframe. |
|start|Date|The start time of the timeframe. |

### TokenPosition

The position of a token in a query string used for errors and notification to map the message to a specific part of the query.

| Name | Type | Description |
| --- | --- | --- |
|end<sup>*required</sup>|<a href="#positioninfo" target="_blank" rel="noopener noreferrer">PositionInfo</a>|The exact position in the query string. |
|start<sup>*required</sup>|<a href="#positioninfo">PositionInfo</a>|The exact position in the query string. |

### VerifyRequest

| Name | Type | Description |
| --- | --- | --- |
|generateCanonicalQuery|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)| <i>default:</i> <code>false</code>|
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The full query string. |
|queryOptions|<a href="#queryoptions" target="_blank" rel="noopener noreferrer">QueryOptions</a>|Query options enhance query functionality for Dynatrace internal services. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |

### VerifyResponse

Verify response.

| Name | Type | Description |
| --- | --- | --- |
|canonicalQuery|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| |
|notifications|Array&lt;<a href="#metadatanotification">MetadataNotification</a>&gt;|The notifications related to the supplied DQL query string. |
|valid<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|True if the supplied DQL query string is valid. |

## Enums

<a name="dqlnodenodetype"></a>
### DQLNodeNodeType

<div class="padding-left--md">

The type of the node.

#### Enum keys

<div class="padding-left--md">

`Alternative` | `Container` | `Terminal`

</div>


</div>

<a name="fieldtypetype"></a>
### FieldTypeType

<div class="padding-left--md">



#### Enum keys

<div class="padding-left--md">

`Array` | `Binary` | `Boolean` | `Double` | `Duration` | `GeoPoint` | `IpAddress` | `Long` | `Record` | `SmartscapeId` | `String` | `Timeframe` | `Timestamp` | `Uid` | `Undefined`

</div>


</div>

<a name="querystate"></a>
### QueryState

<div class="padding-left--md">

Possible state of the query.

#### Enum keys

<div class="padding-left--md">

`Cancelled` | `Failed` | `NotStarted` | `ResultGone` | `Running` | `Succeeded`

</div>


</div>

<a name="tokentype"></a>
### TokenType

<div class="padding-left--md">

The type of the autocomplete token.

#### Enum keys

<div class="padding-left--md">

`Assignment` | `BooleanFalse` | `BooleanTrue` | `BraceClose` | `BraceOpen` | `BracketClose` | `BracketOpen` | `Colon` | `Comma` | `CommandName` | `DataObject` | `Dot` | `EndComment` | `EntityAttribute` | `EntitySelectorPart` | `EntityType` | `FieldModifier` | `FieldPattern` | `FunctionName` | `Indent` | `Linebreak` | `MetricKey` | `Null` | `Number` | `Operator` | `ParameterKey` | `ParameterModifier` | `ParameterValueScope` | `ParenthesisClose` | `ParenthesisOpen` | `ParsePattern` | `Pipe` | `Quote` | `SavedTableName` | `SearchOperator` | `SearchPattern` | `SimpleIdentifier` | `SingleQuote` | `Slash` | `SmartscapeIdValue` | `Space` | `String` | `TimeUnit` | `TimeseriesAggregation` | `TimeseriesAggregationExpression` | `TimestampValue` | `TraversalHopCount` | `TraversalOperator` | `TraversalRelationName` | `UidValue` | `Variable`

</div>


</div>
