import { ErrorResponseDetailsConstraintViolationsItem } from './error-response-details-constraint-violations-item';
import { TokenPosition } from './token-position';
/**
 * Detailed information about the error.
 */
export interface ErrorResponseDetails {
    /**
     * The exception type.
     */
    exceptionType?: string;
    /**
     * The position of a token in a query string used for errors and notification to map the message to a specific part of the query.
     */
    syntaxErrorPosition?: TokenPosition;
    /**
     * The error type, e.g. COMMAND_NAME_MISSING
     */
    errorType?: string;
    /**
     * Complete error message.
     */
    errorMessage?: string;
    /**
     * The arguments for the message format.
     */
    arguments?: string[];
    /**
     * Submitted query string.
     */
    queryString?: string;
    /**
     * The corresponding DQL format specifier types for each format specifier used in the error message format.
     */
    errorMessageFormatSpecifierTypes?: string[];
    /**
     * The message format of the error message, string.format based.
     */
    errorMessageFormat?: string;
    queryId?: string;
    /**
     * Information about an input parameter that violated some validation rule of the service API.
     */
    constraintViolations?: ErrorResponseDetailsConstraintViolationsItem[];
    /**
     * List of missing IAM scopes necessary to successfully execute the request.
     */
    missingScopes?: string[];
    /**
     * List of missing IAM permissions necessary to successfully execute the request.
     */
    missingPermissions?: string[];
}
