import { FilterSegments } from './filter-segments';
import { QueryOptions } from './query-options';
export interface ExecuteRequest {
    /**
     * The full query string.
     */
    query: string;
    /**
     * The query timeframe 'start' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'end' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em>
     */
    defaultTimeframeStart?: string;
    /**
     * The query timeframe 'end' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'start' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em>
     */
    defaultTimeframeEnd?: string;
    /**
     * The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00'
     */
    timezone?: string;
    /**
     * The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'.
     */
    locale?: string;
    /**
     * The maximum number of result records that this query will return.
     */
    maxResultRecords?: number;
    /**
     * The maximum number of result bytes that this query will return.
     */
    maxResultBytes?: number;
    /**
     * The query will stop reading data after reaching the fetch-timeout. The query execution will continue, providing a partial result based on the read data.
     */
    fetchTimeoutSeconds?: number;
    /**
     * The maximum time the response will be delayed to wait for a result. (This excludes the sending time and time spent in any services between the query-frontend and the client.) If the query finishes within the specified timeout, the query result is returned. Otherwise, the requestToken is returned, allowing polling for the result.
     */
    requestTimeoutMilliseconds?: number;
    /**
     * Request preview results. If a preview is available within the requestTimeoutMilliseconds, then it will be returned as part of the response.
     */
    enablePreview?: boolean;
    /**
     * In case not specified in the DQL string, the sampling ratio defined here is applied. Note that this is only applicable to log queries.
     */
    defaultSamplingRatio?: number;
    /**
     * Limit in gigabytes for the amount data that will be scanned during read.
     */
    defaultScanLimitGbytes?: number;
    /**
     * Query options enhance query functionality for Dynatrace internal services.
     */
    queryOptions?: QueryOptions;
    /**
     * Represents a collection of filter segments.
     */
    filterSegments?: FilterSegments;
    /**
     * Parameter to exclude the type information from the query result. In case not specified, the type information will be included.
     */
    includeTypes?: boolean;
}
