import { MetadataNotification } from './metadata-notification';
import { Timeframe } from './timeframe';
/**
 * Collects various bits of metadata information.
 */
export interface GrailMetadata {
    /**
     * The canonical form of the query. It has normalized spaces and canonical constructs.
     */
    canonicalQuery?: string;
    /**
     * Effective timezone for the query.
     */
    timezone?: string;
    /**
     * The submitted query.
     */
    query?: string;
    /**
     * Number of scanned records during the query execution.
     */
    scannedRecords?: number;
    /**
     * The version of DQL that was used to process the query request.
     */
    dqlVersion?: string;
    /**
     * Number of scanned bytes during the query execution.
     */
    scannedBytes?: number;
    scannedDataPoints?: number;
    /**
     * DQL data type timeframe.
     */
    analysisTimeframe?: Timeframe;
    /**
     * Effective locale for the query.
     */
    locale?: string;
    /**
     * The time it took to execute the query.
     */
    executionTimeMilliseconds?: number;
    /**
     * Collected messages during the execution of the query.
     */
    notifications?: MetadataNotification[];
    /**
     * The id of the query
     */
    queryId?: string;
    /**
     * True if sampling was used for at least one segment.
     */
    sampled?: boolean;
}
