export declare function isTextContentType(contentType: string): boolean;
export declare function parseContentDisposition(contentDisposition: string): {
    name: string;
    fileName: string | undefined;
};
export declare function parseFormDataBoundary(contentType: string): string | undefined;
export interface FormDataSection {
    name: string;
    headers: Record<string, string>;
    rawData: Uint8Array;
    text?: string;
    file?: File;
    fileName?: string;
    payloadStartOffset?: number;
    payloadEndOffset?: number;
    headersStartOffset?: number;
}
export declare class FormDataParser {
    private sourceArray;
    private boundary;
    private currentChar;
    private currentByte;
    offset: number;
    private utf8Decoder;
    constructor(sourceArray: Uint8Array, boundary: string);
    get previousByte(): number | undefined;
    get previousChar(): string | undefined;
    private next;
    charAt(offset: number): string | null;
    nextChar(): string | null;
    nextByte(): number | null;
    get nextTwoChars(): string;
    findNextBoundaryOffset(startOffset?: number): number | null;
    skipPastNextBoundary(): boolean;
    readLine(): Uint8Array | null;
    readSectionHeaders(): {
        headersLength: number;
        headers: Record<string, string>;
    };
    extractData(): {
        payloadStartOffset: number;
        payloadEndOffset: number;
        data: Uint8Array;
    };
    parse(): FormDataSection[];
}
export declare function isAppleBrowser(): boolean;
export declare function parseMultipartFormDataResponse(response: Uint8Array, boundary: string): FormDataSection[];
