import { DataTypes } from './data-types';
/**
 * Represents an environment agnostic (browser vs node) abstraction layer for binary data.
 */
export interface Binary {
    /**
     * Returns the binary data in a form that conforms to the specified type.
     *
     * @param type  The type of the data.
     * Can be `text`, `json`, `array-buffer`, `blob`, `readable-stream`, `buffer` or `stream`.
     * @throws {DataTypeError} Data does not conform to the specified type.
     * @throws {UnsupportedOperationError} Type is not supported by the platform.
     */
    get<T extends keyof DataTypes>(type: T): DataTypes[T];
}
