/// <reference types="node" />
/// <reference types="node" />
import type * as stream from 'stream';
import { Binary } from './binary';
/**
 * Represents a plain text field. No filename and content type is set.
 */
export interface TextFormDataRequestField {
    /**
     * A plain text field type discriminator.
     */
    type: 'text';
    /**
     * The name of the field.
     */
    name: string;
    /**
     * The field's value.
     */
    value: string;
}
/**
 * Represents a JSON field.
 */
export interface JsonFormDataRequestField {
    /**
     * A JSON field type discriminator.
     */
    type: 'json';
    /**
     * The name of the field.
     */
    name: string;
    /**
     * The content type of the data contained in `value`.
     * The default content type for the JSON is 'application/json'.
     */
    contentType?: string;
    /**
     * The filename reported to the server.
     * The default filename for the JSON is 'json'.
     */
    filename?: string;
    /**
     * The field's value.
     */
    value: any;
}
/**
 * Represents an binary field.
 */
export interface BinaryFormDataRequestField {
    /**
     * A binary field type discriminator.
     */
    type: 'binary';
    /**
     * The name of the field.
     */
    name: string;
    /**
     * The content type of the data contained in `value`.
     * The default content type for the binary is 'application/octet-stream'.
     */
    contentType?: string;
    /**
     * The filename reported to the server.
     * The default filename for the binary is 'binary'.
     */
    filename?: string;
    /**
     * The field's value.
     */
    value: ArrayBuffer | Blob | ReadableStream | Buffer | stream.Readable | Binary | File;
}
/**
 * Represents a multipart/form-data field.
 */
export type FormDataRequestField = TextFormDataRequestField | JsonFormDataRequestField | BinaryFormDataRequestField;
/**
 * Represents a multipart/form-data body.
 */
export type FormDataRequestBody = Array<FormDataRequestField>;
