import { RequestBodyTypes } from './request-body-types';
import { Headers } from './headers';
import { AbortSignal } from './abort-signal';
import { StatusValidator } from './status-validator';
/**
 * Represents the options used to configure the request.
 */
export interface HttpClientRequestOptions<T extends keyof RequestBodyTypes = 'json'> {
    /**
     * The URL of the resource to fetch.
     */
    url: string;
    /**
     * The request method, e.g. `GET`, `POST`.
     */
    method?: string;
    /**
     * The request headers.
     */
    headers?: Headers;
    /**
     * A request body that will be send along with the request.
     * The structure of the request body needs to conform to the specified request body type.
     */
    body?: RequestBodyTypes[T];
    /**
     * The type of the request body. Can be `text`, `json`, `binary` or `form-data`.
     * The default request body type is `json`.
     */
    requestBodyType?: T;
    /**
     * An AbortSignal instance that can be used to cancel the request.
     */
    abortSignal?: AbortSignal;
    /**
     * A custom status validator function.
     * If the status validator function returns `true` the returned promise resolves with the `HttpClientResponse`
     * Otherwise the promise rejects with the `StatusError`.
     * The default status validator function considers the `2xx` status codes as valid.
     */
    statusValidator?: StatusValidator;
}
