import { ResponseBodyTypes } from './response-body-types';
import { Headers } from './headers';
/**
 * Represents the response to a request.
 */
export interface HttpClientResponse {
    /**
     * The URL of the response.
     */
    readonly url: string;
    /**
     * The status code of the response.
     */
    readonly status: number;
    /**
     * The status message corresponding to the status code.
     */
    readonly statusText: string;
    /**
     * The response headers.
     */
    readonly headers: Headers;
    /**
     * Returns the response body in a form that conforms to the specified response body type.
     *
     * @param responseBodyType  The type of the response body.
     * Can be `text`, `json`, `binary`, `form-data`, `array-buffer`, `blob`, `readable-stream`, `buffer` or `stream`.
     * The default response body type is `json`.
     * @throws {DataTypeError} Response body does not conform to the specified response body type.
     * @throws {UnsupportedOperationError} Response body type is not supported by the platform.
     */
    body<T extends keyof ResponseBodyTypes = 'json'>(responseBodyType?: T): ResponseBodyTypes[T];
}
