import { RequestBodyTypes } from './request-body-types';
import { HttpClientRequestOptions } from './http-client-request-options';
import { HttpClientResponse } from './http-client-response';
/**
 * The http client that serves as an environment agnostic (browser vs node) abstraction for making http calls.
 * It handles the authentication of the requests made to the Platform Services and the Serverless Functions.
 *
 * Currently, only the requests to the Platform Services and the Serverless Functions are supported.
 */
export interface HttpClient {
    /**
     * Sends an http request as configured by the `options`.
     * Returns a promise that resolves with a response object for successful requests or rejects with an appropriate error
     * when one of the following situations happen:
     * - `DataTypeError` - request body does not conform to the specified request body type,
     * - `HttpClientRequestError` - request cannot be sent (e.g. due to network error or invalid request),
     * - `HttpClientResponseError` - response status is considered invalid by the status validator,
     * - `HttpClientAbortError` - request was aborted.
     *
     * @param options  Options used to configure the request.
     * @return Promise<HttpClientResponse> A promise that resolves with the response object or rejects with an error.
     */
    send<T extends keyof RequestBodyTypes = 'json'>(options: HttpClientRequestOptions<T>): Promise<HttpClientResponse>;
}
