/// <reference types="node" />
/// <reference types="node" />
import type * as stream from 'stream';
import { FormDataResponseBody } from './form-data-response-body';
import { Binary } from './binary';
/**
 * Mapping between the response body type names and types.
 */
export type ResponseBodyTypes = {
    /**
     * A text response body type.
     */
    text: Promise<string>;
    /**
     * A JSON response body type.
     */
    json: Promise<any>;
    /**
     * A binary body type.
     */
    binary: Promise<Binary>;
    /**
     * A multipart/form-data body type.
     */
    'form-data': Promise<FormDataResponseBody>;
    /**
     * An ArrayBuffer response body type.
     */
    'array-buffer': Promise<ArrayBuffer>;
    /**
     * A Blob response body type. Browser only.
     */
    blob: Promise<Blob>;
    /**
     * A ReadableStream response body type. Browser only.
     */
    'readable-stream': ReadableStream<Uint8Array>;
    /**
     * A Buffer response body type. Node only.
     */
    buffer: Promise<Buffer>;
    /**
     * A Stream response body type. Node only.
     */
    stream: stream.Readable;
};
