import { ErrorType } from './types';
/**
 * Base error for all client SDKs. All other errors extend this class.
 */
export declare class ApiClientError extends Error {
    readonly cause?: any;
    readonly isApiClientError = true;
    errorType: ErrorType;
    constructor(name: string, message?: string, cause?: any);
}
/**
 * ApiClientError type guard function.
 * Does a structural check of the passed object.
 * @example
 * try {
 *   doSomething();
 * } catch (e: unknown) {
 *   if(isApiClientError(e)) {
 *     handleTheError();
 *   }
 * }
 */
export declare function isApiClientError(e: any): e is ApiClientError;
