import { HttpClientResponse } from '@dynatrace-sdk/http-client';
import { ClientRequestError } from './client-request-error';
import { CommonApiError, CommonErrorDetails, ErrorResponseBody } from './types';
export type ApiGatewayErrorResponseBody = ErrorResponseBody<CommonApiError<ApiGatewayErrorDetails> & {
    retryAfterSeconds?: number;
}>;
/**
 * Reference for *ApiGatewayError* details object. Contains information about received API Getaway error.
 * */
export interface ApiGatewayErrorDetails extends CommonErrorDetails {
    errorCodeProperties?: {
        [key: string]: string;
    };
}
/**
 * Dedicated error response class for errors thrown by API Gateway.
 * Autogenerated SDK Clients have built-in handler for API Gateway errors that throws this error.
 */
export declare class ApiGatewayError extends ClientRequestError<ApiGatewayErrorResponseBody> {
    readonly isApiGatewayError = true;
    readonly code: number;
    readonly retryAfterSeconds: number | undefined;
    constructor(response: HttpClientResponse, body: ApiGatewayErrorResponseBody);
}
/**
 * APIGatewayError type guard function.
 * Does a structural check of the passed object.
 * @example
 * try {
 *   doSomething();
 * } catch (e: unknown) {
 *   if(isApiClientError(e)) {
 *     handleTheError();
 *   }
 * }
 */
export declare function isApiGatewayError(e: any): e is ApiGatewayError;
