import { HttpClientResponse } from '@dynatrace-sdk/http-client';
import { ApiClientError } from './api-client-error';
import { ErrorResponseBody } from './types';
/**
 * Generic error class for service errors, used to handle both expected and unexpected service-level errors.
 * @example Handling unexpected error response, received by the SDK Client
 * const responseValue = await response.body('json');
 *
 * throw new ClientRequestError(
 *   `${response.status}`,
 *   response,
 *   responseValue,
 *   getErrorMessage(
 *     responseValue,
 *     `Unexpected api response: code=${response.status} body="${responseValue}"`,
 *   ),
 *   e,
 * );
 *
 */
export declare class ClientRequestError<DTO = ErrorResponseBody> extends ApiClientError {
    readonly isClientRequestError = true;
    body: DTO;
    response: HttpClientResponse;
    constructor(name: string, response: HttpClientResponse, body: DTO, message?: string, cause?: any);
}
/**
 * ClientRequestError type guard function.
 * Does a structural check of the passed object.
 * @example
 * try {
 *   doSomething();
 * } catch (e: unknown) {
 *   if(isClientRequestError(e)) {
 *     handleTheError();
 *   }
 * }
 */
export declare function isClientRequestError(e: any): e is ClientRequestError;
