import { ApiClientError } from './api-client-error';
/**
 * Dedicated error class for errors related to response serialization.
 * Thrown when received service response can't be deserialized.
 **/
export declare class InvalidResponseError extends ApiClientError {
    readonly isInvalidResponseError = true;
    responseBody: any;
    expectedType?: string;
    nestedError?: Error;
    response: any;
    constructor(name: string, nestedError: Error | any | undefined, body: any, expectedType?: string, message?: string, response?: any);
}
/**
 * InvalidResponseError type guard function.
 * Does a structural check of the passed object.
 * @example
 * try {
 *    doSomething();
 * } catch (e: unknown) {
 *    if (isInvalidResponseError(e)) {
 *      throw e;
 *    }
 * }
 */
export declare function isInvalidResponseError(e: any): e is InvalidResponseError;
